@@ -26,6 +26,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #include "feh.h"
 #include "filelist.h"
+#include "signals.h"
 #include "winwidget.h"
 #include "options.h"
 
@@ -61,7 +62,9 @@ int num_xinerama_screens;
 
 int childpid = 0;
 
+static int feh_file_is_raw(char *filename);
 static char *feh_http_load_image(char *url);
+static char *feh_dcraw_load_image(char *filename);
 static char *feh_magick_load_image(char *filename);
 
 #ifdef HAVE_LIBXINERAMA
@@ -213,8 +216,7 @@ void feh_imlib_print_load_error(char *file, winwidget w, Imlib_Load_Error err)
 int feh_load_image(Imlib_Image * im, feh_file * file)
 {
 	Imlib_Load_Error err = IMLIB_LOAD_ERROR_NONE;
-	enum { SRC_IMLIB, SRC_HTTP, SRC_MAGICK } image_source =
-		SRC_IMLIB;
+	enum { SRC_IMLIB, SRC_HTTP, SRC_MAGICK, SRC_DCRAW } image_source = SRC_IMLIB;
 	char *tmpname = NULL;
 	char *real_filename = NULL;
 
@@ -229,16 +231,23 @@ int feh_load_image(Imlib_Image * im, feh_file * file)
 		if ((tmpname = feh_http_load_image(file->filename)) == NULL)
 			err = IMLIB_LOAD_ERROR_FILE_DOES_NOT_EXIST;
 	}
+	else if (opt.conversion_timeout >= 0 && feh_file_is_raw(file->filename)) {
+		image_source = SRC_DCRAW;
+		tmpname = feh_dcraw_load_image(file->filename);
+		if (!tmpname)
+			err = IMLIB_LOAD_ERROR_NO_LOADER_FOR_FILE_FORMAT;
+	}
 	else
 		*im = imlib_load_image_with_error_return(file->filename, &err);
 
-	if ((err == IMLIB_LOAD_ERROR_UNKNOWN)
-			|| (err == IMLIB_LOAD_ERROR_NO_LOADER_FOR_FILE_FORMAT)) {
+	if (opt.conversion_timeout >= 0 && (
+			(err == IMLIB_LOAD_ERROR_UNKNOWN) ||
+			(err == IMLIB_LOAD_ERROR_NO_LOADER_FOR_FILE_FORMAT))) {
 		image_source = SRC_MAGICK;
 		tmpname = feh_magick_load_image(file->filename);
 	}
 
-	if ((image_source != SRC_IMLIB) && tmpname) {
+	if (tmpname) {
 		*im = imlib_load_image_with_error_return(tmpname, &err);
 		if (!err && im) {
 			real_filename = file->filename;
@@ -286,10 +295,22 @@ int feh_load_image(Imlib_Image * im, feh_file * file)
 	}
 	file->ed = exifData;
 
-	if (orientation == 3)
+	if (orientation == 2)
+		gib_imlib_image_flip_horizontal(*im);
+	else if (orientation == 3)
 		gib_imlib_image_orientate(*im, 2);
+	else if (orientation == 4)
+		gib_imlib_image_flip_vertical(*im);
+	else if (orientation == 5) {
+		gib_imlib_image_orientate(*im, 3);
+		gib_imlib_image_flip_vertical(*im);
+	}
 	else if (orientation == 6)
 		gib_imlib_image_orientate(*im, 1);
+	else if (orientation == 7) {
+		gib_imlib_image_orientate(*im, 3);
+		gib_imlib_image_flip_horizontal(*im);
+	}
 	else if (orientation == 8)
 		gib_imlib_image_orientate(*im, 3);
 #endif
@@ -298,6 +319,95 @@ int feh_load_image(Imlib_Image * im, feh_file * file)
 	return(1);
 }
 
+static int feh_file_is_raw(char *filename)
+{
+	childpid = fork();
+	if (childpid == -1) {
+		perror("fork");
+		return 0;
+	}
+
+	if (childpid == 0) {
+		if (opt.quiet) {
+			int devnull = open("/dev/null", O_WRONLY);
+			dup2(devnull, 1);
+			dup2(devnull, 2);
+		}
+		execlp("dcraw", "dcraw", "-i", filename, NULL);
+		_exit(1);
+	} else {
+		int status;
+		do {
+			waitpid(childpid, &status, WUNTRACED);
+			if (WIFEXITED(status)) {
+				return !WEXITSTATUS(status);
+			}
+		} while (!WIFEXITED(status) && !WIFSIGNALED(status));
+	}
+
+	return 0;
+}
+
+static char *feh_dcraw_load_image(char *filename)
+{
+	char *basename;
+	char *tmpname;
+	char *sfn;
+	int fd = -1;
+
+	basename = strrchr(filename, '/');
+
+	if (basename == NULL)
+		basename = filename;
+	else
+		basename++;
+
+	tmpname = feh_unique_filename("/tmp/", basename);
+
+	if (strlen(tmpname) > (NAME_MAX-6))
+		tmpname[NAME_MAX-7] = '\0';
+
+	sfn = estrjoin("_", tmpname, "XXXXXX", NULL);
+	free(tmpname);
+
+	fd = mkstemp(sfn);
+
+	if (fd == -1) {
+		free(sfn);
+		return NULL;
+	}
+
+	childpid = fork();
+	if (childpid == -1) {
+		weprintf("%s: Can't load with dcraw. Fork failed:", filename);
+		unlink(sfn);
+		free(sfn);
+		close(fd);
+		return NULL;
+	} else if (childpid == 0) {
+
+		close(1);
+		dup(fd);
+		close(fd);
+
+		alarm(opt.conversion_timeout);
+		execlp("dcraw", "dcraw", "-c", "-e", filename, NULL);
+		_exit(1);
+	}
+
+	int status;
+	waitpid(-1, &status, 0);
+	if (WIFSIGNALED(status)) {
+		unlink(sfn);
+		free(sfn);
+		sfn = NULL;
+		if (!opt.quiet)
+			weprintf("%s - Conversion took too long, skipping", filename);
+	}
+
+	return sfn;
+}
+
 static char *feh_magick_load_image(char *filename)
 {
 	char *argv_fn;
@@ -309,9 +419,6 @@ static char *feh_magick_load_image(char *filename)
 	int status;
 	char created_tempdir = 0;
 
-	if (opt.magick_timeout < 0)
-		return NULL;
-
 	basename = strrchr(filename, '/');
 
 	if (basename == NULL)
@@ -387,10 +494,10 @@ static char *feh_magick_load_image(char *filename)
 		_exit(1);
 	}
 	else {
-		alarm(opt.magick_timeout);
+		alarm(opt.conversion_timeout);
 		waitpid(childpid, &status, 0);
 		kill(childpid, SIGKILL);
-		if (opt.magick_timeout > 0 && !alarm(0)) {
+		if (opt.conversion_timeout > 0 && !alarm(0)) {
 			unlink(sfn);
 			free(sfn);
 			sfn = NULL;
@@ -435,6 +542,24 @@ static char *feh_magick_load_image(char *filename)
 
 #ifdef HAVE_LIBCURL
 
+static int curl_quit_function(void *clientp,  curl_off_t dltotal,  curl_off_t dlnow, curl_off_t ultotal, curl_off_t ulnow)
+{
+	// ignore "unused parameter" warnings
+	(void)clientp;
+	(void)dltotal;
+	(void)dlnow;
+	(void)ultotal;
+	(void)ulnow;
+	if (sig_exit) {
+		/*
+		 * The user wants to quit feh. Tell libcurl to abort the transfer and
+		 * return control to the main loop, where we can quit gracefully.
+		 */
+		return 1;
+	}
+	return 0;
+}
+
 static char *feh_http_load_image(char *url)
 {
 	CURL *curl;
@@ -477,12 +602,21 @@ static char *feh_http_load_image(char *url)
 #ifdef DEBUG
 			curl_easy_setopt(curl, CURLOPT_VERBOSE, 1);
 #endif
+			/*
+			 * Do not allow requests to take longer than 30 minutes.
+			 * This should be sufficiently high to accomodate use cases with
+			 * unusually high latencies, while at the sime time avoiding
+			 * feh hanging indefinitely in unattended slideshows.
+			 */
+			curl_easy_setopt(curl, CURLOPT_TIMEOUT, 1800);
 			curl_easy_setopt(curl, CURLOPT_URL, url);
 			curl_easy_setopt(curl, CURLOPT_WRITEDATA, sfp);
 			ebuff = emalloc(CURL_ERROR_SIZE);
 			curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, ebuff);
 			curl_easy_setopt(curl, CURLOPT_FAILONERROR, 1L);
 			curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1);
+			curl_easy_setopt(curl, CURLOPT_XFERINFOFUNCTION, curl_quit_function);
+			curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 0);
 			if (opt.insecure_ssl) {
 				curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, 0);
 				curl_easy_setopt(curl, CURLOPT_SSL_VERIFYHOST, 0);
@@ -495,7 +629,9 @@ static char *feh_http_load_image(char *url)
 			res = curl_easy_perform(curl);
 			curl_easy_cleanup(curl);
 			if (res != CURLE_OK) {
-				weprintf("open url: %s", ebuff);
+				if (res != CURLE_ABORTED_BY_CALLBACK) {
+					weprintf("open url: %s", ebuff);
+				}
 				unlink(sfn);
 				close(fd);
 				free(sfn);
@@ -1098,6 +1234,26 @@ void feh_edit_inplace(winwidget w, int op)
 	if (!w->file || !w->file->data || !FEH_FILE(w->file->data)->filename)
 		return;
 
+	if (!opt.edit) {
+		imlib_context_set_image(w->im);
+		if (op == INPLACE_EDIT_FLIP)
+			imlib_image_flip_vertical();
+		else if (op == INPLACE_EDIT_MIRROR)
+			imlib_image_flip_horizontal();
+		else {
+			imlib_image_orientate(op);
+			tmp = w->im_w;
+			w->im_w = w->im_h;
+			w->im_h = tmp;
+			if (FEH_FILE(w->file->data)->info) {
+				FEH_FILE(w->file->data)->info->width = w->im_w;
+				FEH_FILE(w->file->data)->info->height = w->im_h;
+			}
+		}
+		winwidget_render_image(w, 1, 0);
+		return;
+	}
+
 	if (!strcmp(gib_imlib_image_format(w->im), "jpeg") &&
 			!path_is_url(FEH_FILE(w->file->data)->filename)) {
 		feh_edit_inplace_lossless(w, op);
@@ -1135,8 +1291,12 @@ void feh_edit_inplace(winwidget w, int op)
 		else {
 			imlib_image_orientate(op);
 			tmp = w->im_w;
-			FEH_FILE(w->file->data)->info->width = w->im_w = w->im_h;
-			FEH_FILE(w->file->data)->info->height = w->im_h = tmp;
+			w->im_w = w->im_h;
+			w->im_h = tmp;
+			if (FEH_FILE(w->file->data)->info) {
+				FEH_FILE(w->file->data)->info->width = w->im_w;
+				FEH_FILE(w->file->data)->info->height = w->im_h;
+			}
 		}
 		im_weprintf(w, "unable to edit in place. Changes have not been saved.");
 		winwidget_render_image(w, 1, 0);
