@@ -131,6 +131,7 @@ static void feh_load_options_for_theme(char *theme)
 	char *rcpath = NULL;
 	char *oldrcpath = NULL;
 	char *confbase = getenv("XDG_CONFIG_HOME");
+	// s, s1 and s2 must always have identical size
 	char s[1024], s1[1024], s2[1024];
 	int cont = 0;
 	int bspos;
@@ -167,11 +168,19 @@ static void feh_load_options_for_theme(char *theme)
 		s2[0] = '\0';
 
 		if (cont) {
+			/*
+			 * fgets ensures that s contains no more than 1023 characters
+			 * (+ 1 null byte)
+			 */
 			sscanf(s, " %[^\n]\n", (char *) &s2);
 			if (!*s2)
 				break;
 			D(("Got continued options %s\n", s2));
 		} else {
+			/*
+			 * fgets ensures that s contains no more than 1023 characters
+			 * (+ 1 null byte)
+			 */
 			sscanf(s, "%s %[^\n]\n", (char *) &s1, (char *) &s2);
 			if (!(*s1) || (!*s2) || (*s1 == '\n') || (*s1 == '#')) {
 				cont = 0;
@@ -461,6 +470,8 @@ static void feh_parse_option_array(int argc, char **argv, int finalrun)
 			if (opt.slideshow_delay < 0.0) {
 				opt.slideshow_delay *= (-1);
 				opt.paused = 1;
+			} else {
+				opt.paused = 0;
 			}
 			break;
 		case 'E':
@@ -742,10 +753,12 @@ static void feh_parse_option_array(int argc, char **argv, int finalrun)
 			break;
 		case 234:
 			opt.info_cmd = estrdup(optarg);
-			if (opt.info_cmd[0] == ';')
+			if (opt.info_cmd[0] == ';') {
+				opt.draw_info = 0;
 				opt.info_cmd++;
-			else
+			} else {
 				opt.draw_info = 1;
+			}
 			break;
 		case 235:
 			opt.force_aliasing = 1;
@@ -825,8 +838,6 @@ static void feh_parse_option_array(int argc, char **argv, int finalrun)
 			add_file_to_filelist_recursively(".", FILELIST_FIRST);
 		}
 	}
-	else if (finalrun && !opt.filelistfile && !opt.bgmode)
-		add_file_to_filelist_recursively(".", FILELIST_FIRST);
 
 	/* So that we can safely be called again */
 	optind = 0;
