@@ -1,6 +1,6 @@
 /* signals.c
 
-Copyright (C) 2010 by Daniel Friesel
+Copyright (C) 2010-2018 by Daniel Friesel
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -28,6 +28,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "options.h"
 
 void feh_handle_signal(int);
+volatile int sig_exit = 0;
 
 void setup_signal_handlers()
 {
@@ -86,7 +87,8 @@ void feh_handle_signal(int signo)
 		case SIGQUIT:
 			if (childpid)
 				killpg(childpid, SIGINT);
-			exit(128 + signo);
+			sig_exit = 128 + signo;
+			return;
 	}
 
 	winwid = winwidget_get_first_window_of_type(WIN_TYPE_SLIDESHOW);
