@@ -37,6 +37,24 @@ void init_slideshow_mode(void)
 	int success = 0;
 	gib_list *l = filelist, *last = NULL;
 
+	/*
+	 * In theory, --start-at FILENAME is simple: Look for a file called
+	 * FILENAME, start the filelist there, done.
+	 *
+	 * In practice, there are cases where this isn't sufficient. For instance,
+	 * a user running 'feh --start-at hello.jpg /tmp' will expect feh to start
+	 * at /tmp/hello.jpg, as if they had used
+	 * 'feh --start-at /tmp/hello.jpg /tmp'. Similarly, XDG Desktop files
+	 * may lead to the invocation 'feh --start-at /tmp/hello.jpg .' in /tmp,
+	 * expecting the behaviour of 'feh --start-at ./hello.jpg .'.
+	 *
+	 * Since a good user experience is not about being technically correct, but
+	 * about delivering the expected behaviour, we do some fuzzy matching
+	 * here. In the worst case, this will cause --start-at to start at the
+	 * wrong file.
+	 */
+
+	// Try finding an exact filename match first
 	for (; l && opt.start_list_at; l = l->next) {
 		if (!strcmp(opt.start_list_at, FEH_FILE(l->data)->filename)) {
 			opt.start_list_at = NULL;
@@ -44,6 +62,34 @@ void init_slideshow_mode(void)
 		}
 	}
 
+	/*
+	 * If it didn't work (opt.start_list_at is still set): Fall back to
+	 * comparing just the filenames without directory prefixes. This may lead
+	 * to false positives, but for now that's just the way it is.
+	 */
+	if (opt.start_list_at) {
+		char *current_filename;
+		char *start_at_filename = strrchr(opt.start_list_at, '/');
+		if (start_at_filename) {
+			start_at_filename++; // We only care about the part after the '/'
+		} else {
+			start_at_filename = opt.start_list_at;
+		}
+		for (l = filelist; l && opt.start_list_at; l = l->next) {
+			current_filename = strrchr(FEH_FILE(l->data)->filename, '/');
+			if (current_filename) {
+				current_filename++; // We only care about the part after the '/'
+			} else {
+				current_filename = FEH_FILE(l->data)->filename;
+			}
+			if (!strcmp(start_at_filename, current_filename)) {
+				opt.start_list_at = NULL;
+				break;
+			}
+		}
+	}
+
+	// If that didn't work either, we're out of luck.
 	if (opt.start_list_at)
 		eprintf("--start-at %s: File not found in filelist",
 				opt.start_list_at);
@@ -215,6 +261,7 @@ void feh_reload_image(winwidget w, int resize, int force_new)
 void slideshow_change_image(winwidget winwid, int change, int render)
 {
 	gib_list *last = NULL;
+	gib_list *previous_file = current_file;
 	int i = 0;
 	int jmp = 1;
 	/* We can't use filelist_len in the for loop, since that changes when we
@@ -222,9 +269,12 @@ void slideshow_change_image(winwidget winwid, int change, int render)
 	 */
 	int our_filelist_len = filelist_len;
 
+	if (opt.slideshow_delay > 0.0)
+		feh_add_timer(cb_slide_timer, winwid, opt.slideshow_delay, "SLIDE_CHANGE");
+
 	/* Without this, clicking a one-image slideshow reloads it. Not very *
 	   intelligent behaviour :-) */
-	if (filelist_len < 2 && opt.cycle_once == 0)
+	if (filelist_len < 2 && opt.on_last_slide != ON_LAST_SLIDE_QUIT)
 		return;
 
 	/* Ok. I do this in such an odd way to ensure that if the last or first *
@@ -234,9 +284,11 @@ void slideshow_change_image(winwidget winwid, int change, int render)
 	if (change == SLIDE_FIRST) {
 		current_file = gib_list_last(filelist);
 		change = SLIDE_NEXT;
+		previous_file = NULL;
 	} else if (change == SLIDE_LAST) {
 		current_file = filelist;
 		change = SLIDE_PREV;
+		previous_file = NULL;
 	}
 
 	/* The for loop prevents us looping infinitely */
@@ -252,7 +304,7 @@ void slideshow_change_image(winwidget winwid, int change, int render)
 		case SLIDE_RAND:
 			if (filelist_len > 1) {
 				current_file = feh_list_jump(filelist, current_file, FORWARD,
-					(rand() % (filelist_len - 1)) + 1);
+					(random() % (filelist_len - 1)) + 1);
 				change = SLIDE_NEXT;
 			}
 			break;
@@ -332,6 +384,12 @@ void slideshow_change_image(winwidget winwid, int change, int render)
 			last = NULL;
 		}
 
+		if (opt.on_last_slide == ON_LAST_SLIDE_HOLD && previous_file &&
+			((current_file == filelist && change == SLIDE_NEXT) ||
+			(previous_file == filelist && change == SLIDE_PREV))) {
+				current_file = previous_file;
+		}
+
 		if (winwidget_loadimage(winwid, FEH_FILE(current_file->data))) {
 			int w = gib_imlib_image_get_width(winwid->im);
 			int h = gib_imlib_image_get_height(winwid->im);
@@ -359,8 +417,6 @@ void slideshow_change_image(winwidget winwid, int change, int render)
 	if (filelist_len == 0)
 		eprintf("No more slides in show");
 
-	if (opt.slideshow_delay > 0.0)
-		feh_add_timer(cb_slide_timer, winwid, opt.slideshow_delay, "SLIDE_CHANGE");
 	return;
 }
 
@@ -567,10 +623,10 @@ void feh_filelist_image_remove(winwidget winwid, char do_delete)
 
 		doomed = current_file;
 		/*
-		 * work around feh_list_jump exiting if cycle_once is enabled
+		 * work around feh_list_jump exiting if ON_LAST_SLIDE_QUIT is set
 		 * and no further files are left (we need to delete first)
 		 */
-		if (opt.cycle_once && ! doomed->next && do_delete) {
+		if (opt.on_last_slide == ON_LAST_SLIDE_QUIT && ! doomed->next && do_delete) {
 			feh_file_rm_and_free(filelist, doomed);
 			exit(0);
 		}
@@ -605,16 +661,24 @@ void slideshow_save_image(winwidget win)
 {
 	char *tmpname;
 	Imlib_Load_Error err;
+	char *base_dir = "";
+	if (opt.output_dir) {
+		base_dir = estrjoin("", opt.output_dir, "/", NULL);
+	}
 
 	if (win->file) {
-		tmpname = feh_unique_filename("", FEH_FILE(win->file->data)->name);
+		tmpname = feh_unique_filename(base_dir, FEH_FILE(win->file->data)->name);
 	} else if (mode) {
 		char *tmp;
 		tmp = estrjoin(".", mode, "png", NULL);
-		tmpname = feh_unique_filename("", tmp);
+		tmpname = feh_unique_filename(base_dir, tmp);
 		free(tmp);
 	} else {
-		tmpname = feh_unique_filename("", "noname.png");
+		tmpname = feh_unique_filename(base_dir, "noname.png");
+	}
+
+	if (opt.output_dir) {
+		free(base_dir);
 	}
 
 	if (opt.verbose)
@@ -646,7 +710,7 @@ gib_list *feh_list_jump(gib_list * root, gib_list * l, int direction, int num)
 			if (ret->next) {
 				ret = ret->next;
 			} else {
-				if (opt.cycle_once) {
+				if (opt.on_last_slide == ON_LAST_SLIDE_QUIT) {
 					exit(0);
 				}
 				if (opt.randomize) {
