@@ -203,7 +203,6 @@ void winwidget_create_window(winwidget ret, int w, int h)
 	}
 
 	if (opt.paused) {
-		printf("name %s\n", ret->name);
 		tmpname = estrjoin(" ", ret->name, "[Paused]", NULL);
 		free(ret->name);
 		ret->name = tmpname;
@@ -740,8 +739,6 @@ void winwidget_show(winwidget winwid)
 void winwidget_move(winwidget winwid, int x, int y)
 {
 	if (winwid && ((winwid->x != x) || (winwid->y != y))) {
-		winwid->x = x;
-		winwid->y = y;
 		winwid->x = (x > scr->width) ? scr->width : x;
 		winwid->y = (y > scr->height) ? scr->height : y;
 		XMoveWindow(disp, winwid->win, winwid->x, winwid->y);
